"""
functions for plotting data related to SDOF models
"""
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.io as pio
pio.renderers.default = "browser"  # <- determines how plots are displayed using Plotly


# plot velocity of each vehicle
# can be a single model run or a list of runs
# fill between will fill the space between the first and last runs

def plot_fdx(run_list, run_colors = False, fill_diff = False, show_legend = True):
    """
    run_list can be a single SDOF model run or a list of multiple runs
    colors must be in 'rgb(0, 0, 0)' format
    """

    # test if user supplied a list of runs to plot
    if isinstance(run_list, list):
        plot_list = run_list
    else:
        plot_list = [run_list]  # if not a list, make it a list of length 1

    if run_colors:
        color_list = run_colors
    else:
        color_list = ['rgb(0, 26, 51)'] * len(plot_list)

    # set line types
    line_type = ['solid'] * len(plot_list)
    line_type[0] = 'dash'
    line_type[-1] = 'dashdot'

    # create plot for each run in list
    fig = go.Figure()
    for i, run in enumerate(plot_list):
        fig.add_trace(go.Scatter(x = run.model.dx * -12, y = run.model.springF,
                                mode = 'lines',
                                name = f"{run.name} - Closing Speed: {run.veh1.vx_initial - run.veh2.vx_initial} mph",
                                line = dict(color = color_list[i], width = 2.5, dash = line_type[i])
                                ))

    # fill spaces between first and last trace for each Vehicle
    if fill_diff:
        fig.add_trace(go.Scatter(x = plot_list[0].model.dx * -12, y = plot_list[0].model.springF,
                                mode = 'lines',
                                line = dict(color = color_list[0], width = 2.5, dash = line_type[0]),
                                name = f"{run.name} - Closing Speed: {run.veh1.vx_initial - run.veh2.vx_initial} mph",
                                fill = None,
                                showlegend = False
                                ))
        fig.add_trace(go.Scatter(x = plot_list[-1].model.dx * -12, y = plot_list[-1].model.springF,
                                mode = 'lines',
                                line = dict(color = color_list[-1], width = 2.5, dash = line_type[-1]),
                                name = f"{run.name} - Closing Speed: {run.veh1.vx_initial - run.veh2.vx_initial} mph",
                                fill = 'tonexty',
                                showlegend = False
                                ))

    fig.update_layout(
        legend = dict(orientation = "v", yanchor = 'top', y = 1, xanchor = 'left', x = 0.01),
        autosize = False,
        width = 1600,
        height = 900,
        title = f'Single Degree of Freedom Model - Mutual Force - Displacement',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Mutual Crush (in)'),
        font = dict(family = 'Arial', size = 28, color = 'black'))

    fig.update_layout(showlegend = show_legend)
    fig.update_yaxes(showgrid = False, showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, title_text = 'Force (lb)',
                     rangemode = "tozero", tickfont=dict(family='Arial', size=24))
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside", rangemode = "tozero",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(family='Arial', size=24))
    fig.show()

def plot_vehicle_fdx(run_list, veh1_colors = False, veh2_colors = False, fill_diff = False, show_legend = True):
    """
    run_list can be a single SDOF model run or a list of multiple runs
    veh1_colors and veh2_colors can be lists of colors of the same length of run_list
    colors must be in 'rgb(0, 0, 0)' format
    """

    # test if user supplied a list of runs to plot
    if isinstance(run_list, list):
        plot_list = run_list
    else:
        plot_list = [run_list]  # if not a list, make it a list of length 1

    if veh1_colors:
        veh1_color_list = veh1_colors
    else:
        veh1_color_list = ['rgb(51, 51, 153)'] * len(plot_list)

    if veh2_colors:
        veh2_color_list = veh2_colors
    else:
        veh2_color_list = ['rgb(0, 153, 51)'] * len(plot_list)

    # set line types
    line_type = ['solid'] * len(plot_list)
    line_type[0] = 'dash'
    line_type[-1] = 'dashdot'

    # create plot for each run in list
    fig = go.Figure()
    for i, run in enumerate(plot_list):
        if run.k1known:
            fig.add_trace(go.Scatter(x = run.model.veh1_dx * -12, y = run.model.springF,
                                    mode = 'lines',
                                    name = f"{run.name} - V1",
                                    line = dict(color = veh1_color_list[i], width = 2.5, dash = line_type[i])
                                    ))
            if fill_diff:
                fig.add_trace(go.Scatter(x = plot_list[0].model.veh1_dx * -12, y = plot_list[0].model.springF,
                                        mode = 'lines',
                                        line = dict(color = veh1_color_list[0], width = 2.5, dash = line_type[0]),
                                        name = f"{run.name} - V1",
                                        fill = None,
                                        showlegend = False
                                        ))
                fig.add_trace(go.Scatter(x = plot_list[-1].model.veh1_dx * -12, y = plot_list[-1].model.springF,
                                        mode = 'lines',
                                        line = dict(color = veh1_color_list[-1], width = 2.5, dash = line_type[-1]),
                                        name = f"{run.name} - V1",
                                        fill = 'tonexty',
                                        showlegend = False
                                        ))

        if run.k2known:
            fig.add_trace(go.Scatter(x = run.model.veh2_dx * -12, y = run.model.springF,
                                    mode = 'lines',
                                    name = f"{run.name} - V2",
                                    line = dict(color = veh2_color_list[i], width = 2.5, dash = line_type[i])
                                    ))
            if fill_diff:
                fig.add_trace(go.Scatter(x = plot_list[0].model.veh2_dx * -12, y = plot_list[0].model.springF,
                                        mode = 'lines',
                                        line = dict(color = veh2_color_list[0], width = 2.5, dash = line_type[0]),
                                        name = f"{run.name} - V2",
                                        fill = None,
                                        showlegend = False
                                        ))
                fig.add_trace(go.Scatter(x = plot_list[-1].model.veh2_dx * -12, y = plot_list[-1].model.springF,
                                        mode = 'lines',
                                        line = dict(color = veh2_color_list[-1], width = 2.5, dash = line_type[-1]),
                                        name = f"{run.name} - V2",
                                        fill = 'tonexty',
                                        showlegend = False
                                        ))

    fig.update_layout(
        legend = dict(orientation = "v", yanchor = 'top', y = 1, xanchor = 'left', x = 0.01),
        autosize = False,
        width = 1600,
        height = 900,
        title = f'Single Degree of Freedom Model - Vehicle Crush',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Crush (in)'),
        font = dict(family = 'Arial', size = 28, color = 'black'))

    fig.update_layout(showlegend = show_legend)
    fig.update_yaxes(showgrid = False, showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, title_text = 'Force (lb)',
                     rangemode = "tozero")
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False,
                     rangemode = "tozero")
    fig.show()

def plot_velocity(run_list, veh1_colors = False, veh2_colors = False, fill_diff = False, show_legend = True):
    """
    run_list can be a single SDOF model run or a list of multiple runs
    veh1_colors and veh2_colors can be lists of colors of the same length of run_list
    colors must be in 'rgb(0, 0, 0)' format
    """

    # test if user supplied a list of runs to plot
    if isinstance(run_list, list):
        plot_list = run_list
    else:
        plot_list = [run_list]  # if not a list, make it a list of length 1

    if veh1_colors:
        veh1_color_list = veh1_colors
    else:
        veh1_color_list = ['rgb(51, 51, 153)'] * len(plot_list)

    if veh2_colors:
        veh2_color_list = veh2_colors
    else:
        veh2_color_list = ['rgb(0, 153, 51)'] * len(plot_list)

    # set line types
    line_type = ['solid'] * len(plot_list)
    line_type[0] = 'dash'
    line_type[-1] = 'dash'

    # create plot for each run in list
    fig = go.Figure()
    for i, run in enumerate(plot_list):
        fig.add_trace(go.Scatter(x = run.model.t, y = run.model.v1 * 0.681818,
                                mode = 'lines',
                                name = f"{run.name} - {run.veh1.name}",
                                line = dict(color = veh1_color_list[i], width = 2, dash = line_type[i])
                                ))
        fig.add_trace(go.Scatter(x = run.model.t, y = run.model.v2 * 0.681818,
                                mode = 'lines',
                                name = f"{run.name} - {run.veh2.name}",
                                line = dict(color = veh2_color_list[i], width = 2, dash = line_type[i])
                                ))

    # fill spaces between first and last trace for each Vehicle
    if fill_diff:
        fig.add_trace(go.Scatter(x = plot_list[0].model.t, y = plot_list[0].model.v1 * 0.681818,
                                mode = 'lines',
                                line = dict(color = veh1_color_list[0], width = 1, dash = line_type[0]),
                                name = f"{run.name} - {run.veh1.name}",
                                fill = None,
                                showlegend = False
                                ))
        fig.add_trace(go.Scatter(x = plot_list[-1].model.t, y = plot_list[-1].model.v1 * 0.681818,
                                mode = 'lines',
                                line = dict(color = veh1_color_list[-1], width = 1, dash = line_type[-1]),
                                name = f"{run.name} - {run.veh1.name}",
                                fill = 'tonexty',
                                showlegend = False
                                ))
        fig.add_trace(go.Scatter(x = plot_list[0].model.t, y = plot_list[0].model.v2 * 0.681818,
                                mode = 'lines',
                                line = dict(color = veh2_color_list[0], width = 1, dash = line_type[0]),
                                name = f"{run.name} - {run.veh2.name}",
                                fill = None,
                                showlegend = False
                                ))
        fig.add_trace(go.Scatter(x = plot_list[-1].model.t, y = plot_list[-1].model.v2 * 0.681818,
                                mode = 'lines',
                                line = dict(color = veh2_color_list[-1], width = 1, dash = line_type[-1]),
                                name = f"{run.name} - {run.veh2.name}",
                                fill = 'tonexty',
                                showlegend = False
                                ))

    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.1, xanchor = 'left', x = 0.01),
        autosize = False,
        width = 1600,
        height = 900,
        title = f'Single Degree of Freedom Model - Vehicle Velocity',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Time (s)'),
        font = dict(family = 'Arial', size = 28, color = 'black'))

    fig.update_layout(showlegend = show_legend)
    fig.update_yaxes(showgrid = False, showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, title_text = 'Velocity (mph)', tickfont=dict(family='Arial', size=28))
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(family='Arial', size=28))
    fig.show()

def plot_acceleration(run_list, veh1_colors = False, veh2_colors = False, fill_diff = False, show_legend = True):
    """
    run_list can be a single SDOF model run or a list of multiple runs
    veh1_colors and veh2_colors can be lists of colors of the same length of run_list
    colors must be in 'rgb(0, 0, 0)' format
    """

    # test if user supplied a list of runs to plot
    if isinstance(run_list, list):
        plot_list = run_list
    else:
        plot_list = [run_list]  # if not a list, make it a list of length 1

    if veh1_colors:
        veh1_color_list = veh1_colors
    else:
        veh1_color_list = ['rgb(51, 51, 153)'] * len(plot_list)

    if veh2_colors:
        veh2_color_list = veh2_colors
    else:
        veh2_color_list = ['rgb(0, 153, 51)'] * len(plot_list)

    # set line types
    line_type = ['solid'] * len(plot_list)
    line_type[0] = 'dash'
    line_type[-1] = 'dash'

    # create plot for each run in list
    fig = go.Figure()
    for i, run in enumerate(plot_list):
        fig.add_trace(go.Scatter(x = run.model.t, y = run.model.a1 / 32.2,
                                mode = 'lines',
                                name = f"{run.name} - {run.veh1.name}",
                                line = dict(color = veh1_color_list[i], width = 2, dash = line_type[i])
                                ))
        fig.add_trace(go.Scatter(x = run.model.t, y = run.model.a2 / 32.2,
                                mode = 'lines',
                                name = f"{run.name} - {run.veh2.name}",
                                line = dict(color = veh2_color_list[i], width = 2, dash = line_type[i])
                                ))

    # fill spaces between first and last trace for each Vehicle
    if fill_diff:
        fig.add_trace(go.Scatter(x = plot_list[0].model.t, y = plot_list[0].model.a1 / 32.2,
                                mode = 'lines',
                                line = dict(color = veh1_color_list[0], width = 1, dash = line_type[0]),
                                name = f"{run.name} - {run.veh1.name}",
                                fill = None,
                                showlegend = False
                                ))
        fig.add_trace(go.Scatter(x = plot_list[-1].model.t, y = plot_list[-1].model.a1 / 32.2,
                                mode = 'lines',
                                line = dict(color = veh1_color_list[-1], width = 1, dash = line_type[-1]),
                                name = f"{run.name} - {run.veh1.name}",
                                fill = 'tonexty',
                                showlegend = False
                                ))
        fig.add_trace(go.Scatter(x = plot_list[0].model.t, y = plot_list[0].model.a2 / 32.2,
                                mode = 'lines',
                                line = dict(color = veh2_color_list[0], width = 1, dash = line_type[0]),
                                name = f"{run.name} - {run.veh2.name}",
                                fill = None,
                                showlegend = False
                                ))
        fig.add_trace(go.Scatter(x = plot_list[-1].model.t, y = plot_list[-1].model.a2 / 32.2,
                                mode = 'lines',
                                line = dict(color = veh2_color_list[-1], width = 1, dash = line_type[-1]),
                                name = f"{run.name} - {run.veh2.name}",
                                fill = 'tonexty',
                                showlegend = False
                                ))

    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.1, xanchor = 'left', x = 0.01),
        autosize = False,
        width = 1600,
        height = 900,
        title = f'Single Degree of Freedom Model - Vehicle Acceleration',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Time (s)'),
        font = dict(family = 'Arial', size = 28, color = 'black'))

    fig.update_layout(showlegend = show_legend)
    fig.update_yaxes(showgrid = False, showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, title_text = 'Acceleration (g)')
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()



"""
Use this script to create a new project and directory for storing Pycrash files
Directory Structure:

|-- ProjectName
    |--data
        |--archive      <- stores Pyrcrash compressed data in .pkl format
        |--input        <- place input files here, input created by user will be saved here as .csv files
        |--results      <- model results stored here by Pycrash in .csv format
    |--notebooks        <- store .py or Jupyter Notebook files used to run your Pycrash project
    |--reports          <- used to consolidate output files, user generated reports
    |--visualization    <- Pycrash plots saved here
"""


from cookiecutter.main import cookiecutter
cookiecutter('https://github.com/joe-cormier/pycrash-template/')




import numpy as np

# Spring force from two lists
def SpringFdx(dx, closing, k, input_k_disp, input_k_force, kreturn, dxperm):
    """
    spring force using lookup table of displacement [ft], force [lb]
    input_k_disp from model - [ft]
    input_k_force from input - [lb]
    k - unused
    """
    if dx >= 0:
        return 0

    if closing == 1:
        return np.interp(-1*dx, input_k_disp, input_k_force)

    if (closing == 0) & ((dx - dxperm) < 0): # seperating and dx has not reached dxperm
        return kreturn * abs(dx - dxperm)

    if (closing == 0) & ((dx - dxperm) >= 0): # seperating and dx is less than dxperm
        return 0

# calculate force based on current mutual crush
def SpringForce(dx, closing, k, input_k_disp, input_k_force, kreturn, dxperm):
    """
    Spring force using linear spring assumption
    k - spring stiffness [lb/ft]
    input_k_disp - unused
    input_k_force - unused
    """
    if dx >= 0:
        return 0

    if closing == 1:
        return k * abs(dx)

    if (closing == 0) & ((dx - dxperm) < 0): # closing and dx has not reached dxperm
        return kreturn * abs(dx - dxperm)

    if (closing == 0) & ((dx - dxperm) >= 0): # closing and dx is less than dxperm
        return 0



"""
plots used witin vehicle class
"""
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.io as pio
pio.renderers.default = "browser"  # <- determines how plots are displayed using Plotly

# TODO: create input for figure size - loads from "defaults" folder?
figure_size = (1900, 1200)
font_size = 24
tick_font_size = 22

"""
generating impact point - striking vehicle
"""

def plot_impact_points(veh, user_loc = False, iplane = True):
    # x,y coordinates of vehicle outline:
    # left front corner
    veh._b_lfc_x = veh.lcgf + veh.f_hang
    veh._b_lfc_y = -1 * veh.width / 2
    # right front corner
    veh._b_rfc_x = veh.lcgf + veh.f_hang
    veh._b_rfc_y = veh.width / 2
    # right rear corner
    veh._b_rrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_rrc_y = veh.width / 2
    # left rear corner
    veh._b_lrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_lrc_y = -1* veh.width / 2

    bdy_x = (veh._b_lfc_x, veh._b_rfc_x, veh._b_rrc_x, veh._b_lrc_x, veh._b_lfc_x)
    bdy_y = (veh._b_lfc_y, veh._b_rfc_y, veh._b_rrc_y, veh._b_lrc_y, veh._b_lfc_y)

    x_axis_length = veh._b_lfc_x * 1.5 - veh._b_lrc_x * 1.5

    # generate plot to show vehicle outline and default points for impact
    fig = go.Figure()

    # body outline
    fig.add_trace(go.Scatter(x = bdy_x, y = bdy_y,
                            mode = 'lines+markers',
                            line = dict(color = 'rgb(67,67,67)', width = 2),
                            marker = dict(color = 'rgb(255, 0, 0)', size = 8)))
    # cg
    fig.add_trace(go.Scatter(x = [0,0], y = [0,0],
                            mode='markers',
                            marker = dict(color = 'rgb(51, 204, 51)', size = 20)))

    fig.add_annotation(x = 1,
                       y = -1,
                       showarrow = False,
                       text = "CG")
    # vehicle x-axis
    fig.add_annotation(x = 5,
                       y = 0,
                       axref = 'x',
                       ayref = 'y',
                       text = "",
                       showarrow = True,
                       ax = 0,
                       ay = 0,
                       arrowsize = 2,
                       arrowhead = 1,
                       arrowwidth = 1.5,
                       arrowcolor = 'rgb(0, 0, 0)')
    # vehicle y-axis
    fig.add_annotation(x = 0,
                       y = 5,
                       axref = 'x',
                       ayref = 'y',
                       text = "",
                       showarrow = True,
                       ax = 0,
                       ay = 0,
                       arrowsize = 2,
                       arrowhead = 1,
                       arrowwidth = 1.5,
                       arrowcolor = 'rgb(0, 0, 255)')

    if (user_loc):
        fig.add_trace(go.Scatter(x = [veh.pimpact_x, veh.pimpact_x,], y = [veh.pimpact_y, veh.pimpact_y,],
                            mode = 'markers+text',
                            marker = dict(color = 'rgb(255, 0, 0)', size = 8),
                            text = ['Impact Point', ""],
                            textposition = 'bottom center'))

    else:
        fig.add_annotation(x = veh._b_lfc_x * 1.1,
                           y = veh._b_lfc_y,
                           showarrow = False,
                           text = "1")
        fig.add_annotation(x = veh._b_rfc_x * 1.1,
                           y = veh._b_rfc_y,
                           showarrow = False,
                           text = "2")
        fig.add_annotation(x = veh._b_rrc_x * 1.1,
                           y = veh._b_rrc_y,
                           showarrow = False,
                           text = "3")
        fig.add_annotation(x = veh._b_lrc_x * 1.1,
                           y = veh._b_lrc_y,
                           showarrow = False,
                           text = "4")

    if (iplane):
        # normal impact plane
        fig.add_annotation(x = veh.impact_norm_x,
                           y = veh.impact_norm_y,
                           axref = 'x',
                           ayref = 'y',
                           text = "",
                           showarrow = True,
                           ax = veh.pimpact_x,
                           ay = veh.pimpact_y,
                           arrowsize = 2,
                           arrowhead = 2,
                           arrowwidth = 1.5,
                           arrowcolor = 'rgb(153, 0, 51)')
        # tangential impact plane
        fig.add_annotation(x = veh.impact_tang_x,
                           y = veh.impact_tang_y,
                           axref = 'x',
                           ayref = 'y',
                           text = "",
                           showarrow = True,
                           ax = veh.pimpact_x,
                           ay = veh.pimpact_y,
                           arrowsize = 1,
                           arrowhead = 1,
                           arrowwidth = 1,
                           arrowcolor = 'rgb(153, 0, 51)')

    fig.update_layout(
        showlegend=False,
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title = 'Selecting Impact Point for Striking Vehicle',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'x-axis - Forward (ft)'),
        yaxis = dict(showgrid = False, title = 'y-axis - Rightward (ft)'),
        font = dict(family = 'Arial', size = 16, color = 'black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, range = [veh._b_lrc_x * 1.5, veh._b_lfc_x * 1.5])
    fig.update_yaxes(autorange = 'reversed', showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False,
                     range = [-0.5 * x_axis_length * figure_size[1] / figure_size[0],
                             0.5 * x_axis_length * figure_size[1] / figure_size[0]])
    fig.show()


    """
    generating impacting edge - struck vehicle
    """

def plot_impact_edge(veh):
    # x,y coordinates of vehicle outline:
    # left front corner
    veh._b_lfc_x = veh.lcgf + veh.f_hang
    veh._b_lfc_y = -1 * veh.width / 2
    # right front corner
    veh._b_rfc_x = veh.lcgf + veh.f_hang
    veh._b_rfc_y = veh.width / 2
    # right rear corner
    veh._b_rrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_rrc_y = veh.width / 2
    # left rear corner
    veh._b_lrc_x = -1 * veh.lcgr - veh.r_hang
    veh._b_lrc_y = -1* veh.width / 2

    bdy_x = (veh._b_lfc_x, veh._b_rfc_x, veh._b_rrc_x, veh._b_lrc_x, veh._b_lfc_x)
    bdy_y = (veh._b_lfc_y, veh._b_rfc_y, veh._b_rrc_y, veh._b_lrc_y, veh._b_lfc_y)

    x_axis_length = veh._b_lfc_x * 1.5 - veh._b_lrc_x * 1.5

    # generate plot to show vehicle outline and default points for impact
    fig = go.Figure()

    # body outline
    fig.add_trace(go.Scatter(x = bdy_x[:2], y = bdy_y[:2],
                            mode = 'lines+text',
                            line = dict(color = 'rgb(0, 0, 0)', width = 2)))
    fig.add_trace(go.Scatter(x = bdy_x[1:3], y = bdy_y[1:3],
                            mode = 'lines+text',
                            line = dict(color = 'rgb(51, 102, 204)', width = 2)))
    fig.add_trace(go.Scatter(x = bdy_x[2:4], y = bdy_y[2:4],
                            mode = 'lines+text',
                            line = dict(color = 'rgb(0, 153, 51)', width = 2)))
    fig.add_trace(go.Scatter(x = bdy_x[3:5], y = bdy_y[3:5],
                            mode = 'lines+text',
                            line = dict(color = 'rgb(255, 102, 0)', width = 2)))

    # cg
    fig.add_trace(go.Scatter(x = [0,0], y = [0,0],
                            mode='markers',
                            marker = dict(color = 'rgb(51, 204, 51)', size = 20)))

    # edge labels
    fig.add_annotation(x = veh._b_lfc_x * 1.1,
                       y = 0,
                       showarrow = False,
                       text = "1")
    fig.add_annotation(x = -2,
                       y = 1.2 * veh.width / 2,
                       showarrow = False,
                       text = "2")
    fig.add_annotation(x = veh._b_rrc_x * 1.1,
                       y = 0,
                       showarrow = False,
                       text = "3")
    fig.add_annotation(x = -2,
                       y = -1.2 * veh.width / 2,
                       showarrow = False,
                       text = "4")


    fig.add_annotation(x = 1,
                       y = -1,
                       showarrow = False,
                       text = "CG")
    fig.add_annotation(x = 5,
                       y = 0,
                       axref = 'x',
                       ayref = 'y',
                       text = "",
                       showarrow = True,
                       ax = 0,
                       ay = 0,
                       arrowsize = 2,
                       arrowhead = 1,
                       arrowwidth = 1.5,
                       arrowcolor = 'rgb(0, 0, 0)')

    fig.add_annotation(x = 0,
                       y = 5,
                       axref = 'x',
                       ayref = 'y',
                       text = "",
                       showarrow = True,
                       ax = 0,
                       ay = 0,
                       arrowsize = 2,
                       arrowhead = 1,
                       arrowwidth = 1.5,
                       arrowcolor = 'rgb(0, 0, 255)')



    fig.update_layout(
        showlegend=False,
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title = 'Selecting Impact Edge for Struck Vehicle',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'x-axis - Forward (ft)'),
        yaxis = dict(showgrid = False, title = 'y-axis - Rightward (ft)'),
        font = dict(family = 'Arial', size = 16, color = 'black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, range = [veh._b_lrc_x * 1.5, veh._b_lfc_x * 1.5])
    fig.update_yaxes(autorange = 'reversed', showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False,
                     range = [-0.5 * x_axis_length * figure_size[1] / figure_size[0],
                             0.5 * x_axis_length * figure_size[1] / figure_size[0]])
    fig.show()


"""
plot driver inputs
"""
def plot_driver_inputs(self):
    fig = make_subplots(specs=[[{"secondary_y": True}]])

    fig.add_trace(go.Scatter(x = self.driver_input.t, y = self.driver_input.throttle * 100,
                            mode = 'lines',
                            name = 'throttle',
                            line = dict(color = 'rgb(0, 255, 0)', width = 2)),
                            secondary_y = False)
    fig.add_trace(go.Scatter(x = self.driver_input.t, y = self.driver_input.brake * 100,
                            mode = 'lines',
                            name = 'brake',
                            line = dict(color = 'rgb(255, 0, 0)', width = 2)),
                            secondary_y = False)
    fig.add_trace(go.Scatter(x = self.driver_input.t, y = self.driver_input.steer,
                            mode = 'lines',
                            name = 'steer',
                            line = dict(color = 'rgb(0, 0, 0)', width = 2)),
                            secondary_y = True)

    fig.update_layout(
        legend = dict(orientation = "h", yanchor = 'top', y = 1.05, xanchor = 'left', x = 0.45),
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title = f'Driver Inputs for {self.name}',
        template = 'plotly_white',
        xaxis = dict(showgrid = False, title = 'Time (s)'),
        yaxis = dict(showgrid = False),
        font = dict(family = 'Arial', size = font_size, color = 'black'))

    fig.update_yaxes(showgrid = False, title_text = 'Steer Angle (deg)', secondary_y = True)
    fig.update_yaxes(showgrid = False, title_text = 'Brake | Throttle (%)', secondary_y = False)

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(size=tick_font_size))
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(size=tick_font_size))
    fig.show()



"""
Functions for loading external validation data to compare to pycrash
output
validation data should be placed in /data/input/ project directory
"""

from pandas import import pd
import os

# load valdiation data and rename columns
project_dir = os.chdir(os.path.dirname(os.getcwd()))
input_dir = os.path.join(project_dir, 'data', 'input')


filename = 'validation_data.csv'
original = pd.read_csv(os.path.join(input_dir, filename), na_filter = False)

# get list of column names
orginal_columns = original.columns.values()

# create dictionary to change column names to match pycrash convention
new_column_names = [] # list of new names that corrispond to old names

change_names = dict(zip(orginal_columns, new_column_names))
original.rename(columns = change_names, inplace = True)

# %% Create Vehicle instance with validation vehicle properties

vehicle_input_dict = {"year":2016,
"make":"Subaru",
"model":"WRX Sti",
"weight":3000,
"vin":"123abc",
"brake":0,
"steer_ratio":16.5,
"init_x_pos":0,
"init_y_pos":0,
"head_angle":0,
"width":6.6,
"length":20.66,
"hcg":2,
"lcgf":5.6,
"lcgr":7.76,
"wb":13.36,
"track":5.7,
"f_hang":3.2,
"r_hang":3.873,
"tire_d":2.716666667,
"tire_w":0.866666667,
"izz":3711,
"fwd":0,
"rwd":1,
"awd":0,
"A":100,
"B":41,
"k":1000,
"L":0,
"c":0,
"vx_initial":10,
"vy_initial":0,
"omega_z":0}

val_veh = Vehicle('Veh1', vehicle_input_dict)

# %% Create driver input, or upload from .csv file
end_time = 5  # 5 second simulation
t = list(np.arange(0, end_time+0.1, 0.1))  # create time array from 0 to end time from user
throttle = [0] * len(t)                                # no throttle
brake = [0] * len(t)                                   # no braking
steer = [0] * len(t)                                   # no steering
driver_input_dict = {'t':t, 'throttle':throttle, 'brake':brake, 'steer':steer}
driver_input_df = pd.DataFrame.from_dict(driver_input_dict)

val_veh.driver_input = driver_input_df




import math
import numpy as np
# frontal barrier test:
# generating constant stiffness crash plot


def crash_plot(test_data, veh):
    """
    Derive vehicle stiffness values (A, B) from crash test data
    see Stuble for detailed explanation
    Struble, D. (2014). Automotive Accident Reconstruction Practices and Principles. Boca Raton (FL), CRC Press.
    """
    c_list = test_data['test_crush']
    if max(c_list) == 0:  # create orinate for zero crush
        beta = 1
        CE = 0.5 * (veh.weight / 32.2) * (test_data['test_speed'] * 1.46667)**2 * (1 - test_data['epsilon']**2)  # [ft-lb]
        ECF = math.sqrt(2 * beta * CE * 12 / test_data['damage_length'])  # [Energy Crush Factor]
        return 0, ECF
    else:
        c_ave = 1 / 10 * (c_list[0] + 2 * c_list[1] + 2 * c_list[2] + 2 * c_list[3] + 2 * c_list[4] + c_list[5])
        # form factor
        beta = 1 / (15 * c_ave**2) * (c_list[0]**2 + c_list[0] * c_list[1] + 2 * c_list[1]**2 +
                                             c_list[1] * c_list[2] + 2 * c_list[2]**2 + c_list[2] * c_list[3] +
                                             2 * c_list[3]**2 + c_list[3] * c_list[4] + 2 * c_list[4]**2 +
                                             c_list[4] * c_list[5] + c_list[5]**2)

        CE = 0.5 * (veh.weight / 32.2) * (test_data['test_speed'] * 1.46667)**2 * (1 - test_data['epsilon']**2)  # [ft-lb]



        # crash plot points
        x = beta * c_ave
        ECF = math.sqrt(2 * beta * CE * 12 / test_data['damage_length'])  # [Energy Crush Factor - converting CE to in-lb]

        print(f'average crush = {c_ave}, form factor = {beta}, crush energy = {CE}')
        return x, ECF



"""
repository for all variables used in pycrash modulues
"""
variable_definitions = {
'delta_deg':'tire steer angle (degrees)',
'delta_rad':'tire steer angle (radians)',
'beta_deg':'vehicle slip angle (degrees)',
'beta_rad':'vehicle slip angle (radians)',
'theta_deg':'vehicle heading angle (degrees)',
'theta_rad':'vehicle heading angle (radians)',
'vehicleslip_deg':'vehicle slip angle (degrees)',
'vehicleslip_rad':'vehicle slip angle (radians)',
'lf_alpha':'left front tire slip angle in tire reference frame (radians)',
'rf_alpha':'right front tire slip angle in tire reference frame (radians)',
'rr_alpha':'right rear tire slip angle in tire reference frame (radians)',
'lr_alpha':'left rear tire slip angle in tire reference frame (radians)'
}

def define(variable):
    print(f"{variable} is defined as: )
    return variable_definitions['variable']



"""
functions for plotting data related to vehicle kinematics
"""

import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.io as pio

pio.renderers.default = "browser"  # <- determines how plots are displayed using Plotly

figure_size = (1200, 700)
font_size = 18
"""
vehicle motion - single vehicle
"""


def plot_model(self):
    """
    takes SingleMotion class variable as input
    """

    fig = make_subplots(rows=2, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.05)
    # velocity data
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.Vx / 1.46667,
                             mode='lines',
                             name='X',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.Vy / 1.46667,
                             mode='lines',
                             name='Y',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.Vr / 1.46667,
                             mode='lines',
                             name='Resultant',
                             line=dict(color='rgb(0, 0, 0)', width=2)),
                  row=1, col=1)
    # acceleration data
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.Ax / 32.2,
                             showlegend=False,
                             mode='lines',
                             name='Ax',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.Ay / 32.2,
                             showlegend=False,
                             mode='lines',
                             name='Ay',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.Ar / 32.2,
                             showlegend=False,
                             mode='lines',
                             name='Ar',
                             line=dict(color='rgb(0, 0, 0)', width=2)),
                  row=2, col=1)

    fig.update_layout(
        legend=dict(orientation="h", yanchor='top', y=1.1, xanchor='left', x=0.01),
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title=f'{self.name} - Inertial Frame Velocity | Acceleration',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='Time (s)'),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=2, col=1)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Velocity (mph)', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Acceleration (g)', row=2, col=1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()

    fig = make_subplots(rows=2, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.05)

    # velocity data in vehicle frame
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.vx / 1.46667,
                             mode='lines',
                             name='x',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.vy / 1.46667,
                             mode='lines',
                             name='y',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=1, col=1)

    # acceleration data
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.ax / 32.2,
                             showlegend=False,
                             mode='lines',
                             name='ax',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.ay / 32.2,
                             showlegend=False,
                             mode='lines',
                             name='ay',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=2, col=1)

    fig.update_layout(
        legend=dict(orientation="h", yanchor='top', y=1.1, xanchor='left', x=0.01),
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title=f'{self.name} - Vehicle Frame Velocity | Acceleration',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='Time (s)'),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=2, col=1)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Velocity (mph)', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Acceleration (g)', row=2, col=1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()

    fig = make_subplots(rows=2, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.05)

    # forward tire forces
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.lf_fx,
                             mode='lines',
                             name='LF',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.rf_fx,
                             mode='lines',
                             name='RF',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.rr_fx,
                             mode='lines',
                             name='RR',
                             line=dict(color='rgb(153, 0, 204)', width=2)),
                  row=1, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.lr_fx,
                             mode='lines',
                             name='LR',
                             line=dict(color='rgb(255, 102, 0)', width=2)),
                  row=1, col=1)

    # rightward tire forces
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.lf_fy,
                             showlegend=False,
                             mode='lines',
                             name='LF',
                             line=dict(color='rgb(0, 0, 255)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.rf_fy,
                             showlegend=False,
                             mode='lines',
                             name='RF',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.rr_fy,
                             showlegend=False,
                             mode='lines',
                             name='RR',
                             line=dict(color='rgb(153, 0, 204)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.lr_fy,
                             showlegend=False,
                             mode='lines',
                             name='LR',
                             line=dict(color='rgb(255, 102, 0)', width=2)),
                  row=2, col=1)

    fig.update_layout(
        legend=dict(orientation="h", yanchor='top', y=1.1, xanchor='left', x=0.01),
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title=f'{self.name} - Tire Forces',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='Time (s)'),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=2, col=1)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Forward (lb)', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Rightward (lb)', row=2, col=1)
    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()

    fig = make_subplots(rows=2, cols=1,
                        shared_xaxes=True,
                        vertical_spacing=0.07)

    # heading
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.theta_deg,
                             mode='lines',
                             name='heading',
                             line=dict(color='rgb(0, 0, 0)', width=2)),
                  row=1, col=1)

    # rotational
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.oz_deg,
                             mode='lines',
                             name='omega',
                             line=dict(color='rgb(0, 255, 0)', width=2)),
                  row=2, col=1)
    fig.add_trace(go.Scatter(x=self.model.t, y=self.model.alphaz_deg,
                             mode='lines',
                             name='alpha',
                             line=dict(color='rgb(204, 0, 0)', width=2)),
                  row=2, col=1)

    fig.update_layout(
        legend=dict(orientation="h", yanchor='top', y=1.1, xanchor='left', x=0.01),
        autosize=False,
        width=figure_size[0],
        height=figure_size[1],
        title=f'{self.name} Heading and Rotation',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='Time (s)'),
        yaxis=dict(showgrid=False),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showgrid=False, title_text='Time (s)', row=2, col=1)
    fig.update_xaxes(showgrid=False, title_text='', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Heading (deg)', row=1, col=1)
    fig.update_yaxes(showgrid=False, title_text='Omega (deg/s), Alpha (deg/s/s)', row=2, col=1)

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False)
    fig.show()



# Load NHTSA asii data file
"""
when downloading the ascii data from NHTSA, it will include a .EV5 file, this had the channel info
"""

import os
import pickle
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy import integrate
from pycrash.functions.CFCFilter import cfcfilt


test_num = 10146
data_dir = 'C:\\Users\\jcormier.BIOCORE\\Desktop\\NHTSAData\\v10146ascii'  # will be also used to save data
format_file = f'v{test_num}.EV5'
impact_velocity = 57.09  # test impact speed [kph]

english_units = False
if english_units:
    accel_convert = 32.2
    impact_velocity = impact_velocity * 0.911344  # convert kph to fps
else:
    accel_convert = 9.81
    impact_velocity = impact_velocity * 0.277778  # convert kph to m/s

# find lines with instrumentation info
with open(os.path.join(data_dir, format_file)) as myFile:
    for num, line in enumerate(myFile, 1):
        if 'INSTRUMENTATION' in line:
            print(f'Instrumentation data found at line: {num}')
            instStart = num
        elif '- END -' in line:
            print(f'Instrumentation data found at line: {num}')
            instEnd = num

# find info for barrier data Fx
test_data = {}  # <- dictionary of load cell data
with open(os.path.join(data_dir, format_file)) as myFile:
    for num, line in enumerate(myFile, 1):
        # search for load cell barrier channels
        if (num > instStart) & (num < instEnd) & ('Fx' in line) & ('BARRIER' in line):
            #print(line.split('|'))
            channel_name = line.split('|')[-1].replace('\n','')
            channel_num = int(line.split('|')[1])
            lc_row = channel_name[8]
            lc_col = int(channel_name[10:12])
            print(f'Channel: {channel_name} #{channel_num} at row: {lc_row}, col: {lc_col}')
            test_data[channel_name] = {'type':'LC',
                                       'num':channel_num,
                                       'row':lc_row,
                                       'col':lc_col,
                                       'fileName':f'v{test_num}.{channel_num}'}
        # search for vehicle crossmember acceleration data Ax
        elif (num > instStart) & (num < instEnd) & ('REAR SEAT CROSSMEMBER X' in line):
            channel_name = line.split('|')[-1].replace('\n','')
            channel_num = int(line.split('|')[1])
            print(f'Channel: {channel_name} #{channel_num}')
            if channel_num > 99:
                fileName = f'v{test_num}.{channel_num}'
            elif (channel_num <= 99) & (channel_num > 9):
                fileName = f'v{test_num}.0{channel_num}'
            elif channel_num <= 9:
                fileName = f'v{test_num}.00{channel_num}'
            test_data[channel_name] = {'type':'Accel',
                                       'name':channel_name,
                                       'num':channel_num,
                                       'loc':'Crossmember',
                                       'dir':'x',
                                       'fileName':fileName}


# load data and load cell data for desired rows
row_list = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K']
for key, value in test_data.items():
    if value['type'] == 'LC':
        if (value['row'] in row_list) & ('No valid data' in key):
            print(f'No valid data: {key}')
            test_data[key]['colName'] = f'{value["row"]}_{value["col"]}'
            test_data[key]['time'] = timeData
            test_data[key]['rawforce'] = [0] * len(timeData) # set force to zero if missing
            test_data[key]['rawforce'] = [0] * len(timeData) # set force to zero if missing
            test_data[key]['force'] = [0] * len(timeData)
        elif (value['row'] in row_list) & ('No valid data' not in key):
            print(f'loading file {value["fileName"]}')
            #df = pd.read_csv(os.path.join(data_dir, value["fileName"]), names=['time','data'], sep='\t')
            # load file with numpy
            data_array = np.loadtxt(os.path.join(data_dir, value["fileName"]), delimiter='\t')
            all_time = list(data_array[:, 0])
            all_data = list(data_array[:, 1])
            zero_time_index = all_time.index(0)
            time = all_time[zero_time_index:]
            data = all_data[zero_time_index:]
            test_data[key]['colName'] = f'{value["row"]}_{value["col"]}'
            test_data[key]['time'] = time
            test_data[key]['rawforce'] = data
            test_data[key]['force'] = cfcfilt(60, data, time[1]-time[0])
            timeData = time
    elif value['type'] == 'Accel':
        #df = pd.read_csv(os.path.join(data_dir, value["fileName"]), names=['time', 'data'], sep='\t')
        data_array = np.loadtxt(os.path.join(data_dir, value["fileName"]), delimiter='\t')
        all_time = list(data_array[:, 0])
        all_data = list(data_array[:, 1])
        zero_time_index = all_time.index(0)
        time = all_time[zero_time_index:]
        data = all_data[zero_time_index:]
        test_data[key]['colName'] = value['name']
        test_data[key]['time'] = time
        test_data[key]['rawAccel'] = data
        filtered_accel = cfcfilt(60, data, time[1] - time[0])
        test_data[key]['Accel'] = filtered_accel # filtered acceleration [g]
        accel_ = [x * accel_convert for x in filtered_accel]
        velocity = integrate.cumtrapz(accel_, time, initial=0)
        velocity = [x + impact_velocity for x in velocity]
        #displacement = integrate.cumtrapz(velocity, time, initial=0)
        test_data[key]['Velocity'] = velocity  # velocity [fps / m/s]
        test_data[key]['Displacement'] = integrate.cumtrapz(velocity, time, initial=0)
        #test_data[key]['Displacement'] = [x - displacement[0] for x in displacement]  # 0 at 0 displacement [ft / mm]


# create dictionary of filtered force
forceDict = {}
forceDict['time'] = test_data[next(iter(test_data))]['time']
for key, value in test_data.items():
    if value['type'] == 'LC':
        #print(value['colName'])
        forceDict[value['colName']] = value['force']  # filtered force data

# convert to dataframe
forceDF = pd.DataFrame.from_dict(forceDict)
# fix columns with missing data
for label, content in forceDF.items():
    if sum(content) == 0:
        print(f'Column {label} = 0')

forceDF.F_1 = forceDF.F_2
forceDF.C_4 = np.mean([forceDF.C_3, forceDF.C_5])

# sum by row
row_list = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K']
column_list = list(np.arange(1, 17))

for row in row_list:
    sum_columns = [row + "_" + str(s) for s in column_list]
    forceDF[f'{row}_Sum'] = forceDF[sum_columns].sum(axis=1)

# get velocity / displacement data
AccelDict = {}
AccelDict['time'] = test_data[next(iter(test_data))]['time']
for key, value in test_data.items():
    if value['type'] == 'Accel':
        print(value['colName'])
        AccelDict[value['colName']] = value['Accel']  # filtered acceleration [g]
        AccelDict[f"{value['colName']} Velocity"] = value['Velocity']  # velocity [fps]
        AccelDict[f"{value['colName']} Disp"] = value['Displacement']  # displacement [ft]

AccelDF = pd.DataFrame.from_dict(AccelDict)

AccelDF.plot(x='time', y = 'LEFT REAR SEAT CROSSMEMBER X', figsize=(12,7))
plt.show()
AccelDF.plot(x='time', y = 'LEFT REAR SEAT CROSSMEMBER X Velocity', figsize=(12,7))
plt.show()
AccelDF.plot(x='time', y = 'LEFT REAR SEAT CROSSMEMBER X Disp', figsize=(12,7))
plt.show()


# save
with open(os.path.join(data_dir, f'ProcessedNHTSATestData_{test_num}.pkl'), 'wb') as handle:
    pickle.dump(test_data, handle, protocol=pickle.HIGHEST_PROTOCOL)

# %% Combined data by columns -
# sum by row
rowForce = {}
rowForce['time'] = test_data[next(iter(test_data))]['time']
rowSum = [0] * len(test_data[next(iter(test_data))]['time'])
currentRow = 'K'
for key, value in test_data.items():
    if value['type'] == 'LC':
        if value['row'] == currentRow:
            rowSum = [a + b for a, b in zip(rowSum, value['force'])]
            rowForce[value['row']] = rowSum
        else:
            rowSum = [0] * len(test_data[next(iter(test_data))]['time'])
            rowSum = [a + b for a, b in zip(rowSum, value['force'])]
            rowForce[value['row']] = rowSum
            currentRow = value['row']


RowForcedf = pd.DataFrame.from_dict(rowForce)


# merge force and displacement df
Fdx = pd.merge(RowForcedf, AccelDF, how = 'left', on='time')
Fdx.to_pickle(os.path.join(data_dir, f'Force_DisplacementNHTSA_Test_{test_num}.pkl'))

Fdx.plot(x='time', y = 'LEFT REAR SEAT CROSSMEMBER X', figsize=(12,7))
plt.show()


""" 
generate square wave acceleration profile for free
particle analyses
"""

# %% modules
import os
os.chdir('D:\\OneDrive\\pycrash')

from data.defaults.config import default_dict
import matplotlib.pyplot as plt
from scipy import integrate
import pandas as pd
import numpy as np

dt_impact = default_dict['dt_impact']            # iteration time step

# %% reate input columns
end_time = 0.136                # time duration seconds
constant_accel = -1 * 32.2      # constant acceleration (f/s/s)
v_initial = 60 * 1.46667        # initial speed (fps)
x_initial = 0                   # initial position (ft)

t = list(np.arange(0, dt_impact + end_time, dt_impact))  # create time array from 0 to max time in inputs, this will be end time for simulation
t = [round(x, 4) for x in t]
a = [constant_accel] * len(t)
v = v_initial + integrate.cumtrapz(a, t, initial = 0)     # integrate accel to get velocity
x = x_initial + integrate.cumtrapz(v, t, initial = 0)     # integrate velocity to get position

# %% calculate free particle relative displacement

def relative_disp(t, x, v_initial):
    x_relative = [None] * len(t)
    for i in range(len(t)):
        free_part_x = v_initial * t[i]
        x_relative[i] = free_part_x - x[i]
    
    return x_relative

# %% Create range of runs
accel_list = [-0.5 * 32.2, -0.75 * 32.2, -1 * 32.2] # list for model runs   
x_list = [None] * len(accel_list)
v_list = [None] * len(accel_list)
x_relative_list = [None] * len(accel_list)

for i in range(len(accel_list)):
    a = [accel_list[i]] * len(t)
    v_list[i] = v_initial + integrate.cumtrapz(a, t, initial = 0)
    x_list[i] = x_initial + integrate.cumtrapz(v_list[i], t, initial = 0)
    x_relative_list[i] = relative_disp(t, x_list[i], v_initial)

# %% Plot Velocity
plt.figure(figsize = (16,9))
#plt.title('Velocity', fontsize=20)
color_list = ['b', 'g', 'k']

for i in range(len(accel_list)):
    v_mph = [x / 1.46667 for x in v_list[i]]
    print(f'Total Change in Speed = {v_mph[len(v_mph)-1] - v_mph[0]:0.2f} mph')
    print(f'Final Speed = {v_mph[len(v_mph)-1]:0.2f} mph')
    print("")
    plt.plot(t, v_mph, label = f'Accel = {accel_list[i]/32.2:0.2f} g', c = color_list[i])  

plt.xticks(fontsize=16)
plt.yticks(fontsize=16)
plt.xlim([0, max(t)])
plt.ylim([round(min(v_mph), 1) - 0.5, round(max(v_mph), 1)])
ax = plt.gca()
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
plt.xlabel('Time(s)', fontsize=20)
plt.ylabel('Velocity (mph)', fontsize=20)
plt.legend(fontsize=16, frameon = False)
plt.show()


# %% Plot Relative Displacement
plt.figure(figsize = (16,9))
#plt.title('Relative Displacement', fontsize=20)
color_list = ['b', 'g', 'k']

for i in range(len(accel_list)):
    x_rel_inches = [x * 12 for x in x_relative_list[i]]
    print(f'Total Relative Motion = {x_rel_inches[len(x_rel_inches)-1]:0.1f} in')
    print("")
    plt.plot(t, x_rel_inches, label = f'Accel = {accel_list[i]/32.2:0.2f} g', c = color_list[i])  

plt.xticks(fontsize=16)
plt.yticks(fontsize=16)
plt.xlim([0, max(t)])
plt.ylim([0, round(max(x_rel_inches), 1) + 0.5])
ax = plt.gca()
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
plt.xlabel('Time(s)', fontsize=20)
plt.ylabel('Relative Displacement (inch)', fontsize=20)
plt.legend(fontsize=16, frameon = False)
plt.show()



from .project import Project, project_info, load_project
from .vehicle import Vehicle
from .kinematics import SingleMotion
from .kinematicstwo import KinematicsTwo
from .sdof_model import SDOF_Model


__all__ = ["Project", "Vehicle", "SingleMotion", "KinematicsTwo", "SDOF_Model"]



from tabulate import tabulate
from cookiecutter.main import cookiecutter
import os
import pickle

os.chdir(os.path.dirname(os.getcwd()))


# TODO: when loading project, pull saved project data

def yes_or_no(question):
    while "the answer is invalid":
        reply = str(input(question + ' (y/n): ')).lower().strip()
        if reply[:1] == 'y':
            return True
        if reply[:1] == 'n':
            return False


def save_project_data(projectself):
    """
    save project data in pycrash data directory to share project info with other modules
    """
    if os.path.exists(os.path.join(projectself.project_path, projectself.name, "data", "archive", datafileName)):
        over_write_file = yes_or_no("Project file already exists here - overwrite?: ")
        if over_write_file:
            os.remove(
                os.path.join(projectself.project_path, projectself.name, "data", "archive", datafileName))  # delete current file
            ProjectData = project_objects

        else:
            new_project_name = str(input("Enter new project name: "))
            projectself.name = new_project_name
            ProjectData = project_objects


class Project:
    """
    class object for project variables
    name - project name - will be used to save and load project data
    pdesc - brief description of project, impact type, etc.
    sim_type - what type of simulation performed, single or multi vehicle
    impact_type - sideswipe (SS), momentum (IMPC), single degree of freedom (SDOF)
                - defaults to "none"
    note - user note
    """

    def __init__(self, project_input=None):
        if (project_input == None):
            self.name = input("Project Name: ")
            self.project_path = str(input("Enter path to project directory: ")),
            self.pdesc = input("Project Description: ")
            self.sim_type = input("Simulation Type [Single Vehicle = SV | Multi-Vehicle = MV]: ")
            self.type = "project"  # class type

            if self.sim_type == "MV":
                self.impact_type = input("Impact Type (SDOF, SS, IMPC): ")

                if self.impact_type not in ["SS", "IMPC", "SDOF"]:
                    print("Not a valid impact type, choose SS, IMPC or SDOF. Value set to SDOF")
                    self.impact_type == None
            else:
                self.impact_type = None

            self.note = input("Note: ")
        else:
            self.name = project_input['name']
            self.project_path = project_input['project_path']
            self.pdesc = project_input['pdesc']
            self.sim_type = project_input['sim_type']
            self.impact_type = project_input['impact_type']
            self.note = project_input['note']
            self.type = 'project'  # class type

        # check if project directory exists
        # if it does, then a new project data file can be created at project/data/archive
        if not os.path.isdir(os.path.join(self.project_path, self.name)):
            os.chdir(self.project_path)
            cookie_cutter_repo = f'https://github.com/joe-cormier/pycrash.git'
            cookiecutter(cookie_cutter_repo, no_input=True, extra_context={'project_name': self.name})
        else:
            print(f'Project directory {os.path.join(self.project_path, self.name)} already exists')

        print(f'Project directories located here: {os.path.join(self.project_path, self.name)}')
        print(f'Place any input files to be used in {os.path.join(self.project_path, self.name, "data", "input")}')
        print('')

        # save data into archive for notebook reference
        datafileName = ''.join([self.name, '.pkl'])
        project_objects = {}
        project_objects.update({self.name: self})
        # test if datafileName.pkl exists
        if os.path.exists(os.path.join(self.project_path, self.name, "data", "archive", datafileName)):
            over_write_file = yes_or_no("Project file already exists here - overwrite?: ")
            if over_write_file:
                os.remove(
                    os.path.join(self.project_path, self.name, "data", "archive", datafileName))  # delete current file
                ProjectData = project_objects

            else:
                new_project_name = str(input("Enter new project name: "))
                datafileName = ''.join([new_project_name, '.pkl'])
                self.name = new_project_name
                ProjectData = project_objects

        else:
            # create new file for saving project data
            ProjectData = project_objects

        with open(os.path.join(self.project_path, self.name, "data", "archive", datafileName), 'wb') as handle:
            pickle.dump(ProjectData, handle, protocol=pickle.HIGHEST_PROTOCOL)

        print(tabulate([["Project", "Description", "Impact Type", "Simulation Type", "Note"],
                        [self.name, self.pdesc, self.impact_type, self.sim_type, self.note]]))

    def show(self):
        print(tabulate([["Project", "Description", "Impact Type", "Simulation Type", "Note"],
                        [self.name, self.pdesc, self.impact_type, self.sim_type, self.note]]))


    def save_project(self, *args):
        """
        root_path - directory to store project data
        save project to filename along with vehicles of Class Vehicle
        will create a directory and subdirectory for data and reports if
        they don't exist
        TODO: add functionality for 3+ vehicles and model runs
        """
        nvehicles = 0
        nsdof_models = 0
        nsideswipe = 0
        nmultimotion = 0
        nsinglemotion = 0
        project_objects = {}
        project_objects.update({self.name: self})

        for a in args:
            if (a.type == 'vehicle'):
                nvehicles += 1
                project_objects.update({f'veh{nvehicles}': a})
            elif (a.type == 'sdof'):
                nsdof_models += 1
                project_objects.update({f'sdof{nsdof_models}': a})
            elif (a.type == 'sideswipe'):
                nsideswipe += 1
                project_objects.update({f'ss{nsideswipe}': a})
            elif (a.type == 'singlemotion'):
                nsinglemotion += 1
                project_objects.update({f'singlemotion{nsinglemotion}': a})
            elif (a.type == 'multimotion'):
                nmultimotion += 1
                project_objects.update({f'multimotion{nmultimotion}': a})
            else:
                print(f"Unknown object type for {a} of type {type(a)}")

        # filename for data is the project Name
        datafileName = ''.join([self.name, '.pkl'])

        # test if ProjectData.pkl exists
        if os.path.exists(os.path.join(self.project_path, self.name, "data", "archive", datafileName)):
            with open(os.path.join(self.project_path, self.name, "data", "archive", datafileName), 'rb') as handle:
                ProjectData = pickle.load(handle)
                # add new project to data file
                ProjectData = project_objects
        elif os.path.exists(os.path.join(self.project_path, self.name, "data", "archive", datafileName)) == False:
            # create new file for saving project data
            ProjectData = project_objects

        with open(os.path.join(self.project_path, self.name, "data", "archive", datafileName), 'wb') as handle:
            pickle.dump(ProjectData, handle, protocol=pickle.HIGHEST_PROTOCOL)


def project_info(project_name):
    """
    pulls project data to be used when reloading saved data
    """
    datafileName = ''.join([project_name, '.pkl'])
    out_names = []
    print("This saved project contains:")
    with open(os.path.join(os.path.dirname(os.getcwd()), project_name, "data", "archive", datafileName), 'rb') as handle:
        ProjectData = pickle.load(handle)
    for key, value in ProjectData.items():
        print(f'Object of type "{value.type}" with name "{value.name}"')
        out_names.append(value.name)

    print(f'list objects in this order for loading project: {out_names}')
    print(f"Example: project_name, veh1, veh2 = load_project('project_name')")


def load_project(project_name):
    """
    load saved project data using information from "project_info"
    requires multiple variables for input:

    Example prject with two vehicles:
    project, veh1, veh2 = load_project('ProjectName')
    """
    path_parent = os.path.dirname(os.getcwd())
    datafileName = ''.join([project_name, '.pkl'])

    out_data = []
    with open(os.path.join(path_parent, project_name, "data", "archive", datafileName), 'rb') as handle:
        ProjectData = pickle.load(handle)
        value_iterator = iter(ProjectData.values())
        first_value = next(value_iterator)

    if len(ProjectData) == 1:
        return first_value
    else:
        for value in value_iterator:
            out_data.append(value)
        return out_data



# various equations for reconstruction
import numpy as np


def b1values(bo, L, C, W):
    """
    calculate b1 given bo, damage length (L), damage depth (C) and vehicle weight (W)
    calculate b1 and bo values
    bo = damage offset speed (mph)
    b1 = slope of barrier impact speed and residual crush (mph/in)
    L = with of direct damage
    w = vehicle weight
    """

    return -1 * bo / C + np.sqrt((bo**2 + 2))


def ABfrombob1(W, bo, b1, L):
    """
    return A and B stiffness values from Weight, b0, b1 and damage length (L)
    calculate A anb B values from bo and b1
    L = damage length [in]
    W = vehicle weigth [lb]
    bo = damage offset speed [mph]
    b1 = slope of BEV - residual crush [mph/in]
    """

    A = 0.802 * W * bo * b1 / L
    B = 0.802 * W * b1**2 / L
    return [A, B]

def CrushEnergyAB(A, B, L, C):
    """
    crush energy [in-lb] from A and B values
    L - crush length [in]
    C - average crush depth  [in]
    """

    return L * (A*C + (B * C**2 / 2)  + (A**2 / (2*B)))

def CrushForceAB(A, B, L, C):
    """
    calculate force from crush using A and B
    A [lb/in]
    B [lb/in/in]
    L crush length [in]
    C crush depth [in]
    """

    return L * (A + B * C)

def StrikingDV(w1, w2, v1, v2, rest):
    """
    calculate striking vehicle delta-V [mph] given:
    w1, w2 - vehicle weights [lb]
    v1, v2 - striking vehicle speeds [mph]
    rest - restitution
    """
    m1 = w1 / 32.2
    m2 = w2 / 32.2
    v1 = v1 * 1.46667
    v2 = v2 * 1.46667
    return (m2 / (m1 + m2) * (1 + rest) * (v1 - v2)) * 0.681818181818181

def StruckDV(w1, w2, v1, v2, rest):
    """
    calculate struck vehicle delta-V [mph] given:
    w1, w2 - vehicle weights [lb]
    v1, v2 - striking vehicle speeds [mph]
    rest - restitution
    """
    m1 = w1 / 32.2
    m2 = w2 / 32.2
    v1 = v1 * 1.46667
    v2 = v2 * 1.46667
    return (m1 / (m1 + m2) * (1 + rest) * (v1 - v2)) * 0.681818181818181


def EnergyDV(w1, w2, Edis, cor):
    """
    see Rose SAE #2005-01-1200
    vehicle 1 and 2 delta-V [mph]
    Edis = dissipated energy [ft-lb]
    cor = coefficient of restitution
    """

    m1 = w1 / 32.2
    m2 = w2 / 32.2
    A = (m1 * m2) / (m1 + m2)
    B = (1 + cor) / (1 - cor)
    dv1 = (1/m1) * np.sqrt(A * 2 * B * Edis)
    dv2 = (1/m2) * np.sqrt(A * 2 * B * Edis)
    return [dv1*0.681818181818181, dv2*0.681818181818181]

def formFactor(crush_list_mm):
    """
    crush_list is a list of 6 crush measurements [mm]
    """

    c = crush_list_mm * 0.0393701
    cbar = c
    cbar[0] = cbar[0] / 2
    cbar[5] = cbar[5] / 2
    cbar = np.sum(cbar) / 5
    A = 1 / (15 * cbar**2)
    B = c[0]**2 + c[0]*c[1] + 2*c[1]**2 + c[1]*c[2] + 2*c[2]**2 + c[2]*c[3] + 2*c[3]**2 + \
        c[3]*c[4] + 2*c[4]**2 + c[4]*c[5] + c[5]**2
    return A * B

def formFactorin(crush_list_in):
    """
    crush_list is a list of 6 crush measurements [in]
    """

    c = crush_list_in
    cbar = c
    cbar[0] = cbar[0] / 2
    cbar[5] = cbar[5] / 2
    cbar = np.sum(cbar) / 5
    A = 1 / (15 * cbar**2)
    B = c[0]**2 + c[0]*c[1] + 2*c[1]**2 + c[1]*c[2] + 2*c[2]**2 + c[2]*c[3] + 2*c[3]**2 + \
        c[3]*c[4] + 2*c[4]**2 + c[4]*c[5] + c[5]**2
    return A * B

def BarrierCrushEnergy(W, s):
    """
    W [lb] = weight of test vehicle
    s [mph] = barrier impact speed
    """

    return 0.5 * (W/32.2) * (s*1.46667)**2

def cipriani_rest(ClosingSpeed):
    """
    calculate restitution based on closing speed in mph
    based on regression performed by Cipriani
    Cipriani, A., et al. (2002). "Low Speed Collinear Impact Severity: A Comparison Between Full Scale Testing and Analytical Prediction Tools with Restitution Analysis."
    Society of Automotive Engineers 2002-01-0540.
    """

    A = 0.47477
    B = 0.26139 * np.log10(abs(ClosingSpeed) * 0.44704)
    C = 0.03382 * np.log10(abs(ClosingSpeed) * 0.44704)**2
    D = 0.11639 * np.log10(abs(ClosingSpeed) * 0.44704)**3
    return A - B + C - D

def CrushEnergyInt(dx, f):
    """
    calculate dissipated energy from force-deformation data
    dx [ft]
    f [lb]
    """

    return np.trapz(f, x=dx)

def SpringSeriesKeff(k1, k2):
    """
    calcualte effective stiffness for two springs in series
    """

    return 1 / (1/k1 + 1/k2)

def BEVfromE(W, E):
    """
    calculate Barrier Equivalent Velocity [mph] from Energy [ft/lb]
    """
    return np.sqrt(2*E/(W/32.2)) * 0.681818



import plotly.graph_objects as go
import plotly.io as pio
pio.renderers.default = "browser"
font_size = 24
tick_font_size = 24
width = 1900
aspect_ratio = 16 / 9
figure_size = (width, width / aspect_ratio)


def cg_motion(model1, model2, name1, name2):
    dx_max = max(model1.Dx.max(), model2.Dx.max())
    dx_min = min(model1.Dx.min(), model2.Dx.min())
    dy_max = max(model1.Dy.max(), model2.Dy.max())
    dy_min = min(model1.Dy.min(), model2.Dy.min())

    dx = dx_max - dx_min
    dy = dy_max - dy_min

    if dx > dy:
        adj_x = aspect_ratio * dy / dx
        adj_y = 1
        #print(f" dx > dy -> adj_x = {adj_x}, adj_y = {adj_y}")
        dx_min = round(dx_min * adj_x)
        dx_max = round(dx_max * adj_x)
        #print(f"dx_min = {dx_min}, dx_max = {dx_max}")
        #print(f"dy_min = {dy_min}, dy_max = {dy_max}")
    else:
        adj_x = 1
        adj_y = (1 / aspect_ratio) * dx / dy
        #print(f" dy > dx -> adj_x = {adj_x}, adj_y = {adj_y}")
        dy_min = round(dy_min * adj_y)
        dy_max = round(dy_max * adj_y)
        #print(f"dx_min = {dx_min}, dx_max = {dx_max}")
        #print(f"dy_min = {dy_min}, dy_max = {dy_max}")

    """
    plot location of CG in global reference frame
    """
    fig = go.Figure()

    fig.add_trace(go.Scatter(x=model1.Dx, y=model1.Dy,
                             mode='markers',
                             name=f'{name1}',
                             marker=dict(color='rgb(0, 0, 0)', size=3.5),
                             ))
    fig.add_trace(go.Scatter(x=model2.Dx, y=model2.Dy,
                             mode='markers',
                             name=f'{name2}',
                             marker=dict(color='rgb(0, 0, 256)', size=5)
                             ))

    fig.update_layout(
        showlegend=True,
        autosize=False,
        width=width,
        height=width / aspect_ratio,
        title='Vehicle Motion in Global Reference Frame',
        template='plotly_white',
        xaxis=dict(showgrid=False, title='x-axis - Forward (ft)', range = [dx_min, dx_max]),
        yaxis=dict(showgrid=False, title='y-axis - Rightward (ft)', range = [dy_max, dy_min]),
        font=dict(family='Arial', size=font_size, color='black'))

    fig.update_xaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(size=tick_font_size))
    fig.update_yaxes(showline=True, linewidth=1, linecolor='black', ticks="outside",
                     tickwidth=1, tickcolor='black', ticklen=10, zeroline=False, tickfont=dict(size=tick_font_size))
    fig.show()
